@@ -21,10 +21,7 @@
 #ifndef __UP_DAEMON_H__
 #define __UP_DAEMON_H__
 
-#include <glib-object.h>
-#include <polkit/polkit.h>
-#include <dbus/dbus-glib.h>
-
+#include <dbus/up-daemon-generated.h>
 #include "up-types.h"
 #include "up-device-list.h"
 
@@ -41,13 +38,13 @@ typedef struct UpDaemonPrivate UpDaemonPrivate;
 
 typedef struct
 {
-	GObject	parent;
+	UpExportedDaemonSkeleton parent;
 	UpDaemonPrivate	*priv;
 } UpDaemon;
 
 typedef struct
 {
-	GObjectClass		 parent_class;
+	UpExportedDaemonSkeletonClass parent_class;
 } UpDaemonClass;
 
 typedef enum
@@ -60,57 +57,41 @@ typedef enum
 
 #define UP_DAEMON_ERROR up_daemon_error_quark ()
 
-GType up_daemon_error_get_type (void);
-#define UP_DAEMON_TYPE_ERROR (up_daemon_error_get_type ())
-
 GQuark		 up_daemon_error_quark		(void);
 GType		 up_daemon_get_type		(void);
 UpDaemon	*up_daemon_new			(void);
-void		 up_daemon_test		(gpointer	 user_data);
-DBusBusType	 up_daemon_get_bus_type		(void);
-void		 up_daemon_set_bus_type		(DBusBusType		 type);
 
 /* private */
 guint		 up_daemon_get_number_devices_of_type (UpDaemon	*daemon,
 						 UpDeviceKind		 type);
 UpDeviceList	*up_daemon_get_device_list	(UpDaemon		*daemon);
-gboolean	 up_daemon_startup		(UpDaemon		*daemon);
+gboolean	 up_daemon_startup		(UpDaemon		*daemon,
+						 GDBusConnection 	*connection);
+void		 up_daemon_shutdown		(UpDaemon		*daemon);
 void		 up_daemon_set_lid_is_closed	(UpDaemon		*daemon,
 						 gboolean		 lid_is_closed);
 void		 up_daemon_set_lid_is_present	(UpDaemon		*daemon,
 						 gboolean		 lid_is_present);
-void		 up_daemon_set_lid_force_sleep	(UpDaemon		*daemon,
-						 gboolean		 lid_force_sleep);
-void		 up_daemon_set_is_docked	(UpDaemon		*daemon,
-						 gboolean		 is_docked);
 void		 up_daemon_set_on_battery	(UpDaemon		*daemon,
 						 gboolean		 on_battery);
-void		 up_daemon_set_on_low_battery	(UpDaemon		*daemon,
-						 gboolean		 on_low_battery);
+void		 up_daemon_set_warning_level	(UpDaemon		*daemon,
+						 UpDeviceLevel		 warning_level);
+UpDeviceLevel	 up_daemon_compute_warning_level(UpDaemon		*daemon,
+						 UpDeviceState		 state,
+						 UpDeviceKind		 kind,
+						 gboolean		 power_supply,
+						 gdouble		 percentage,
+						 gint64			 time_to_empty);
+const gchar	*up_daemon_get_charge_icon	(UpDaemon		*daemon,
+						 gdouble		 percentage,
+						 UpDeviceLevel		 battery_level,
+						 gboolean		 charging);
 
-/* exported */
-gboolean	 up_daemon_enumerate_devices	(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_get_on_battery	(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_get_low_battery	(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_suspend		(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_about_to_sleep	(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_suspend_allowed	(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_hibernate		(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_hibernate_allowed	(UpDaemon		*daemon,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_can_suspend		(UpDaemon		*daemon,
-						 gboolean		 interactive,
-						 DBusGMethodInvocation	*context);
-gboolean	 up_daemon_can_hibernate	(UpDaemon		*daemon,
-						 gboolean		 interactive,
-						 DBusGMethodInvocation	*context);
+void		 up_daemon_start_poll		(GObject		*object,
+						 GSourceFunc		 callback);
+void		 up_daemon_stop_poll		(GObject		*object);
+void             up_daemon_pause_poll           (UpDaemon               *daemon);
+void             up_daemon_resume_poll          (UpDaemon               *daemon);
 
 G_END_DECLS
 
