@@ -33,10 +33,9 @@
 
 static void	up_history_finalize	(GObject		*object);
 
-#define UP_HISTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), UP_TYPE_HISTORY, UpHistoryPrivate))
-
-#define UP_HISTORY_FILE_HEADER		"PackageKit Profile"
 #define UP_HISTORY_SAVE_INTERVAL	(10*60)		/* seconds */
+#define UP_HISTORY_SAVE_INTERVAL_LOW_POWER	5	/* seconds */
+#define UP_HISTORY_LOW_POWER_PERCENT	10
 #define UP_HISTORY_DEFAULT_MAX_DATA_AGE	(7*24*60*60)	/* seconds */
 
 struct UpHistoryPrivate
@@ -51,8 +50,9 @@ struct UpHistoryPrivate
 	GPtrArray		*data_charge;
 	GPtrArray		*data_time_full;
 	GPtrArray		*data_time_empty;
-	guint			 save_id;
+	GSource			*save_source;
 	guint			 max_data_age;
+	gchar			*dir;
 };
 
 enum {
@@ -60,7 +60,7 @@ enum {
 	UP_HISTORY_LAST_SIGNAL
 };
 
-G_DEFINE_TYPE (UpHistory, up_history, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (UpHistory, up_history, G_TYPE_OBJECT)
 
 /**
  * up_history_set_max_data_age:
@@ -118,18 +118,16 @@ up_history_array_limit_resolution (GPtrArray *array, guint max_num)
 {
 	UpHistoryItem *item;
 	UpHistoryItem *item_new;
-	gfloat division;
 	guint length;
-	gint i;
-	guint last;
-	guint first;
+	guint i;
+	guint64 last;
+	guint64 first;
 	GPtrArray *new;
 	UpDeviceState state = UP_DEVICE_STATE_UNKNOWN;
 	guint64 time_s = 0;
 	gdouble value = 0;
 	guint64 count = 0;
 	guint step = 1;
-	gfloat preset;
 
 	new = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	g_debug ("length of array (before) %i", array->len);
@@ -150,15 +148,14 @@ up_history_array_limit_resolution (GPtrArray *array, guint max_num)
 	item = (UpHistoryItem *) g_ptr_array_index (array, 0);
 	first = up_history_item_get_time (item);
 
-	division = (first - last) / (gfloat) max_num;
-	g_debug ("Using a x division of %f (first=%i,last=%i)", division, first, last);
-
 	/* Reduces the number of points to a pre-set level using a time
 	 * division algorithm so we don't keep diluting the previous
 	 * data with a conventional 1-in-x type algorithm. */
-	for (i=length-1; i>=0; i--) {
+	for (i = 0; i < length; i++) {
+		guint64 preset;
+
 		item = (UpHistoryItem *) g_ptr_array_index (array, i);
-		preset = last + (division * (gfloat) step);
+		preset = last + ((first - last) * (guint64) step) / max_num;
 
 		/* if state changed or we went over the preset do a new point */
 		if (count > 0 &&
@@ -212,6 +209,12 @@ up_history_copy_array_timespan (const GPtrArray *array, guint timespan)
 	if (array->len == 0)
 		return NULL;
 
+	/* no limit on data */
+	if (timespan == 0) {
+		array_new = g_ptr_array_ref ((GPtrArray *) array);
+		goto out;
+	}
+
 	/* new data */
 	array_new = g_ptr_array_new ();
 	g_get_current_time (&timeval);
@@ -224,7 +227,7 @@ up_history_copy_array_timespan (const GPtrArray *array, guint timespan)
 		if (timeval.tv_sec - up_history_item_get_time (item) < timespan)
 			g_ptr_array_add (array_new, g_object_ref (item));
 	}
-
+out:
 	return array_new;
 }
 
@@ -292,7 +295,7 @@ up_history_get_profile_data (UpHistory *history, gboolean charging)
 	g_return_val_if_fail (UP_IS_HISTORY (history), NULL);
 
 	/* create 100 item list and set to zero */
-	data = g_ptr_array_new ();
+	data = g_ptr_array_new_full (101, g_object_unref);
 	for (i=0; i<101; i++) {
 		stats = up_stats_item_new ();
 		g_ptr_array_add (data, stats);
@@ -394,11 +397,22 @@ up_history_get_filename (UpHistory *history, const gchar *type)
 	gchar *filename;
 
 	filename = g_strdup_printf ("history-%s-%s.dat", type, history->priv->id);
-	path = g_build_filename (PACKAGE_LOCALSTATE_DIR, "lib", "upower", filename, NULL);
+	path = g_build_filename (history->priv->dir, filename, NULL);
 	g_free (filename);
 	return path;
 }
 
+/**
+ * up_history_set_directory:
+ **/
+void
+up_history_set_directory (UpHistory *history, const gchar *dir)
+{
+	g_free (history->priv->dir);
+	history->priv->dir = g_strdup (dir);
+	g_mkdir_with_parents (dir, 0755);
+}
+
 /**
  * up_history_array_to_file:
  * @list: a valid #GPtrArray instance
@@ -514,6 +528,8 @@ up_history_array_from_file (GPtrArray *list, const gchar *filename)
 		ret = up_history_item_set_from_string (item, parts[i]);
 		if (ret)
 			g_ptr_array_add (list, item);
+		else
+			g_object_unref (item);
 	}
 
 out:
@@ -574,7 +590,7 @@ static gboolean
 up_history_schedule_save_cb (UpHistory *history)
 {
 	up_history_save_data (history);
-	history->priv->save_id = 0;
+	g_clear_pointer (&history->priv->save_source, g_source_destroy);
 	return FALSE;
 }
 
@@ -602,7 +618,7 @@ up_history_is_low_power (UpHistory *history)
 		return FALSE;
 
 	/* high enough */
-	if (up_history_item_get_value (item) > 10)
+	if (up_history_item_get_value (item) > UP_HISTORY_LOW_POWER_PERCENT)
 		return FALSE;
 
 	/* we are low power */
@@ -616,28 +632,35 @@ static gboolean
 up_history_schedule_save (UpHistory *history)
 {
 	gboolean ret;
+	gint timeout = UP_HISTORY_SAVE_INTERVAL;
 
 	/* if low power, then don't batch up save requests */
 	ret = up_history_is_low_power (history);
 	if (ret) {
-		g_warning ("saving directly to disk as low power");
-		up_history_save_data (history);
-		return TRUE;
+		g_debug ("saving to disk earlier due to low power");
+		timeout = UP_HISTORY_SAVE_INTERVAL_LOW_POWER;
 	}
 
-	/* we already have one saved */
-	if (history->priv->save_id != 0) {
-		g_debug ("deferring as others queued");
-		return TRUE;
+	/* we already have one saved, clear it if it will fire earlier */
+	if (history->priv->save_source) {
+		guint64 ready = g_source_get_ready_time (history->priv->save_source);
+
+		if (ready > g_source_get_time (history->priv->save_source) + timeout * G_USEC_PER_SEC) {
+			g_clear_pointer (&history->priv->save_source, g_source_destroy);
+		} else {
+			g_debug ("deferring as earlier timeout is already queued");
+			return TRUE;
+		}
 	}
 
-	/* nothing scheduled, do new */
-	g_debug ("saving in %i seconds", UP_HISTORY_SAVE_INTERVAL);
-	history->priv->save_id = g_timeout_add_seconds (UP_HISTORY_SAVE_INTERVAL,
-							(GSourceFunc) up_history_schedule_save_cb, history);
-#if GLIB_CHECK_VERSION(2,25,8)
-	g_source_set_name_by_id (history->priv->save_id, "[UpHistory] save");
-#endif
+	/* nothing scheduled */
+	g_debug ("saving in %i seconds", timeout);
+	history->priv->save_source = g_timeout_source_new_seconds (timeout);
+	g_source_set_name (history->priv->save_source, "[upower] up_history_schedule_save_cb");
+	g_source_attach (history->priv->save_source, NULL);
+	g_source_set_callback (history->priv->save_source,
+			       (GSourceFunc) up_history_schedule_save_cb, history,
+			       NULL);
 	return TRUE;
 }
 
@@ -856,7 +879,6 @@ up_history_class_init (UpHistoryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	object_class->finalize = up_history_finalize;
-	g_type_class_add_private (klass, sizeof (UpHistoryPrivate));
 }
 
 /**
@@ -866,17 +888,14 @@ up_history_class_init (UpHistoryClass *klass)
 static void
 up_history_init (UpHistory *history)
 {
-	history->priv = UP_HISTORY_GET_PRIVATE (history);
-	history->priv->id = NULL;
-	history->priv->rate_last = 0;
-	history->priv->percentage_last = 0;
-	history->priv->state = UP_DEVICE_STATE_UNKNOWN;
+	history->priv = up_history_get_instance_private (history);
 	history->priv->data_rate = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	history->priv->data_charge = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	history->priv->data_time_full = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	history->priv->data_time_empty = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
-	history->priv->save_id = 0;
 	history->priv->max_data_age = UP_HISTORY_DEFAULT_MAX_DATA_AGE;
+
+	up_history_set_directory (history, HISTORY_DIR);
 }
 
 /**
@@ -893,8 +912,7 @@ up_history_finalize (GObject *object)
 	history = UP_HISTORY (object);
 
 	/* save */
-	if (history->priv->save_id > 0)
-		g_source_remove (history->priv->save_id);
+	g_clear_pointer (&history->priv->save_source, g_source_destroy);
 	if (history->priv->id != NULL)
 		up_history_save_data (history);
 
@@ -904,6 +922,7 @@ up_history_finalize (GObject *object)
 	g_ptr_array_unref (history->priv->data_time_empty);
 
 	g_free (history->priv->id);
+	g_free (history->priv->dir);
 
 	g_return_if_fail (history->priv != NULL);
 
@@ -918,8 +937,6 @@ up_history_finalize (GObject *object)
 UpHistory *
 up_history_new (void)
 {
-	UpHistory *history;
-	history = g_object_new (UP_TYPE_HISTORY, NULL);
-	return UP_HISTORY (history);
+	return g_object_new (UP_TYPE_HISTORY, NULL);
 }
 
